@@ -80,6 +80,7 @@ gnome_vfs_file_info_fields_get_type (void)
       { GNOME_VFS_FILE_INFO_FIELDS_SYMLINK_NAME, "GNOME_VFS_FILE_INFO_FIELDS_SYMLINK_NAME", "symlink-name" },
       { GNOME_VFS_FILE_INFO_FIELDS_MIME_TYPE, "GNOME_VFS_FILE_INFO_FIELDS_MIME_TYPE", "mime-type" },
       { GNOME_VFS_FILE_INFO_FIELDS_ACCESS, "GNOME_VFS_FILE_INFO_FIELDS_ACCESS", "access" },
+      { GNOME_VFS_FILE_INFO_FIELDS_IDS, "GNOME_VFS_FILE_INFO_FIELDS_IDS", "ids" },
       { 0, NULL, NULL }
     };
     etype = g_flags_register_static ("GnomeVFSFileInfoFields", values);
@@ -469,6 +470,7 @@ gnome_vfs_xfer_options_get_type (void)
       { GNOME_VFS_XFER_USE_UNIQUE_NAMES, "GNOME_VFS_XFER_USE_UNIQUE_NAMES", "use-unique-names" },
       { GNOME_VFS_XFER_LINK_ITEMS, "GNOME_VFS_XFER_LINK_ITEMS", "link-items" },
       { GNOME_VFS_XFER_FOLLOW_LINKS_RECURSIVE, "GNOME_VFS_XFER_FOLLOW_LINKS_RECURSIVE", "follow-links-recursive" },
+      { GNOME_VFS_XFER_TARGET_DEFAULT_PERMS, "GNOME_VFS_XFER_TARGET_DEFAULT_PERMS", "target-default-perms" },
       { 0, NULL, NULL }
     };
     etype = g_flags_register_static ("GnomeVFSXferOptions", values);
