@@ -62,6 +62,7 @@ struct _GnomeVFSVolumePrivate {
 
 	/* Only for HAL devices: */
 	char *hal_udi;
+	char *hal_drive_udi; /* only available to daemon; not exported */
 
 	/* Only for connected servers */
 	char *gconf_id;
@@ -85,17 +86,18 @@ struct _GnomeVFSDrivePrivate {
 
 	/* Only for HAL devices: */
 	char *hal_udi;
+	char *hal_drive_udi; /* only available to daemon; not exported */
 
 	gboolean must_eject_at_unmount;
 };
 
-void _gnome_vfs_volume_set_drive                (GnomeVFSVolume        *volume,
+void gnome_vfs_volume_set_drive_private         (GnomeVFSVolume        *volume,
 						 GnomeVFSDrive         *drive);
-void _gnome_vfs_drive_add_mounted_volume        (GnomeVFSDrive         *drive,
+void gnome_vfs_drive_add_mounted_volume_private (GnomeVFSDrive         *drive,
 						 GnomeVFSVolume        *volume);
-void _gnome_vfs_drive_remove_volume             (GnomeVFSDrive         *drive,
+void gnome_vfs_drive_remove_volume_private      (GnomeVFSDrive         *drive,
 						 GnomeVFSVolume        *volume);
-void _gnome_vfs_volume_unset_drive              (GnomeVFSVolume        *volume,
+void gnome_vfs_volume_unset_drive_private       (GnomeVFSVolume        *volume,
 						 GnomeVFSDrive         *drive);
 void _gnome_vfs_volume_monitor_mounted          (GnomeVFSVolumeMonitor *volume_monitor,
 						 GnomeVFSVolume        *volume);
@@ -138,6 +140,11 @@ GnomeVFSVolume *_gnome_vfs_volume_monitor_find_volume_by_hal_udi (GnomeVFSVolume
 GnomeVFSDrive *_gnome_vfs_volume_monitor_find_drive_by_hal_udi   (GnomeVFSVolumeMonitor *volume_monitor,
 								  const char            *hal_udi);
 
+GnomeVFSVolume *_gnome_vfs_volume_monitor_find_volume_by_hal_drive_udi (GnomeVFSVolumeMonitor *volume_monitor,
+									const char            *hal_drive_udi);
+GnomeVFSDrive *_gnome_vfs_volume_monitor_find_drive_by_hal_drive_udi   (GnomeVFSVolumeMonitor *volume_monitor,
+									const char            *hal_drive_udi);
+
 #endif /* USE_HAL */
 
 GnomeVFSVolume *_gnome_vfs_volume_monitor_find_volume_by_device_path (GnomeVFSVolumeMonitor *volume_monitor,
