@@ -337,7 +337,7 @@ handle_overwrite (GnomeVFSResult *result,
 	switch (*overwrite_mode) {
 	case GNOME_VFS_XFER_OVERWRITE_MODE_ABORT:
 		*replace = FALSE;
-		*result = GNOME_VFS_ERROR_FILE_EXISTS;
+		*result = GNOME_VFS_ERROR_INTERRUPTED;
 		*skip = FALSE;
 		return FALSE;
 	case GNOME_VFS_XFER_OVERWRITE_MODE_REPLACE:
@@ -357,7 +357,7 @@ handle_overwrite (GnomeVFSResult *result,
 		switch (action) {
 		case GNOME_VFS_XFER_OVERWRITE_ACTION_ABORT:
 			*replace = FALSE;
-			*result = GNOME_VFS_ERROR_FILE_EXISTS;
+			*result = GNOME_VFS_ERROR_INTERRUPTED;
 			*skip = FALSE;
 			return FALSE;
 		case GNOME_VFS_XFER_OVERWRITE_ACTION_REPLACE:
@@ -755,10 +755,9 @@ list_add_items_and_size (const GList *name_uri_list,
 	each_params.progress = progress;
 	each_params.result = GNOME_VFS_OK;
 
+	info_options = GNOME_VFS_FILE_INFO_DEFAULT;
 	if (xfer_options & GNOME_VFS_XFER_FOLLOW_LINKS) {
-		info_options = GNOME_VFS_FILE_INFO_FOLLOW_LINKS;
-	} else {
-		info_options = GNOME_VFS_FILE_INFO_DEFAULT;
+		info_options |= GNOME_VFS_FILE_INFO_FOLLOW_LINKS;
 	}
 
 	return gnome_vfs_visit_list (name_uri_list, info_options,
@@ -805,10 +804,9 @@ directory_add_items_and_size (GnomeVFSURI *dir_uri,
 	each_params.progress = progress;
 	each_params.result = GNOME_VFS_OK;
 
+	info_options = GNOME_VFS_FILE_INFO_DEFAULT;
 	if (xfer_options & GNOME_VFS_XFER_FOLLOW_LINKS) {
-		info_options = GNOME_VFS_FILE_INFO_FOLLOW_LINKS;
-	} else {
-		info_options = GNOME_VFS_FILE_INFO_DEFAULT;
+		info_options |= GNOME_VFS_FILE_INFO_FOLLOW_LINKS;
 	}
 
 	return gnome_vfs_directory_visit_uri (dir_uri, info_options,
@@ -939,10 +937,9 @@ handle_merged_directory_name_conflicts (GnomeVFSXferOptions xfer_options,
 	if (xfer_options & GNOME_VFS_XFER_SAMEFS)
 		visit_options |= GNOME_VFS_DIRECTORY_VISIT_SAMEFS;
 
+	info_options = GNOME_VFS_FILE_INFO_DEFAULT;
 	if (xfer_options & GNOME_VFS_XFER_FOLLOW_LINKS) {
-		info_options = GNOME_VFS_FILE_INFO_FOLLOW_LINKS;
-	} else {
-		info_options = GNOME_VFS_FILE_INFO_DEFAULT;
+		info_options |= GNOME_VFS_FILE_INFO_FOLLOW_LINKS;
 	}
 
 	params.result = GNOME_VFS_OK;
@@ -1026,9 +1023,15 @@ handle_name_conflicts (GList **source_uri_list,
 		    gnome_vfs_get_file_info_uri (uri, target_info, GNOME_VFS_FILE_INFO_DEFAULT) == GNOME_VFS_OK) {
 			progress_set_source_target_uris (progress, source_uri, uri);
 			
-			/* no error getting info -- file exists, ask what to do */
-			replace = handle_overwrite (&result, progress, error_mode,
-						    overwrite_mode, &replace, &skip);
+			/* Skip if we are trying to copy the file to itself */
+			if (gnome_vfs_uri_equal (uri, source_uri)) {
+				replace = FALSE;
+				skip = TRUE;
+			} else {
+				/* no error getting info -- file exists, ask what to do */
+				replace = handle_overwrite (&result, progress, error_mode,
+							    overwrite_mode, &replace, &skip);
+			}
 			
 			/* FIXME bugzilla.eazel.com 1207:
 			 * move items to Trash here
@@ -1763,6 +1766,7 @@ copy_items (const GList *source_uri_list,
 		result = gnome_vfs_get_file_info_uri (source_uri, info, 
 						      GNOME_VFS_FILE_INFO_DEFAULT);
 
+		g_free (progress->progress_info->duplicate_name);
 		progress->progress_info->duplicate_name =
 			gnome_vfs_uri_extract_short_path_name
 			((GnomeVFSURI *)target_item->data);
@@ -1909,6 +1913,7 @@ move_items (const GList *source_uri_list,
 		source_uri = (GnomeVFSURI *)source_item->data;
 		target_dir_uri = gnome_vfs_uri_get_parent ((GnomeVFSURI *)target_item->data);
 
+		g_free (progress->progress_info->duplicate_name);
 		progress->progress_info->duplicate_name =  
 			gnome_vfs_uri_extract_short_path_name
 			((GnomeVFSURI *)target_item->data);
@@ -2016,6 +2021,8 @@ link_items (const GList *source_uri_list,
 		source_reference = gnome_vfs_uri_to_string (source_uri, GNOME_VFS_URI_HIDE_NONE);
 
 		target_dir_uri = gnome_vfs_uri_get_parent ((GnomeVFSURI *)target_item->data);
+
+		g_free (progress->progress_info->duplicate_name);
 		progress->progress_info->duplicate_name =
 			gnome_vfs_uri_extract_short_path_name
 			((GnomeVFSURI *)target_item->data);
@@ -2205,6 +2212,7 @@ gnome_vfs_xfer_delete_items (const GList *source_uri_list,
 	 */
 	progress->progress_info->bytes_total
 		= progress->progress_info->files_total * DEFAULT_SIZE_OVERHEAD;
+	progress->progress_info->top_level_item = TRUE;
 	if (result != GNOME_VFS_ERROR_INTERRUPTED) {
 		call_progress (progress, GNOME_VFS_XFER_PHASE_READYTOGO);
 		result = gnome_vfs_xfer_delete_items_common (source_uri_list,
@@ -2516,11 +2524,10 @@ gnome_vfs_xfer_uri_internal (const GList *source_uris,
 		progress->progress_info->total_bytes_copied = 0;
 
 		if (result != GNOME_VFS_OK) {
-			/* don't care about any results from handle_error */
-			handle_error (&result, progress, &error_mode, &skip);
-
-			/* whatever error it was, we handled it */
-			result = GNOME_VFS_OK;
+			if (handle_error (&result, progress, &error_mode, &skip)) {
+				/* whatever error it was, we handled it */
+				result = GNOME_VFS_OK;
+			}
 		} else {
 			call_progress (progress, GNOME_VFS_XFER_PHASE_READYTOGO);
 
@@ -2650,25 +2657,25 @@ _gnome_vfs_xfer_private (const GList *source_uri_list,
 
 /**
  * gnome_vfs_xfer_uri_list:
- * @source_uri_list: A Glist of uris  (ie file;//)
- * @target_uri_list: A GList of uris  
+ * @source_uri_list: a #GList of uris  (ie file://).
+ * @target_uri_list: a #GList of uris.
  * @xfer_options: These are options you wish to set for the transfer. For
  * instance by setting the xfer behavior you can either make a copy or a 
  * move.  
  * @error_mode:  Decide how to behave if the xfer is interrupted.  For instance
  * you could set your application to return an error code in case of an
- * interuption.
- * @overwrite_mode: How to react if a file your copying is being overwritten.
+ * interruption.
+ * @overwrite_mode: How to react if a file is being overwritten.
  * @progress_callback:  This is used to monitor the progress of a transfer.
  * Common use would be to check to see if the transfer is asking for permission
  * to overwrite a file.
- * @data: Data to be want passed back in callbacks from the xfer engine
+ * @data: Data to be passed back in callbacks from the xfer engine.
  *
- * This function will transfer multiple files to a multiple targets.  Given a
- * a source uri(s) and a destination uri(s).   There are a list of options that
+ * This function will transfer multiple files to a multiple targets.  Given
+ * source uri(s) and destination uri(s).   There are a list of options that
  * your application can use to control how the transfer is done.
  *
- * Returns: If all goes well it returns GNOME_VFS_OK.  Check GnomeVFSResult for
+ * Returns: If all goes well it returns %GNOME_VFS_OK.  Check #GnomeVFSResult for
  * other values.
  **/
 GnomeVFSResult
@@ -2725,19 +2732,19 @@ gnome_vfs_xfer_uri_list (const GList *source_uri_list,
  * GNOME_VFS_XFER_RECURSIVE xfer option.
  * GNOME_VFS_XFER_REMOVESOURCE: This option will remove the source data after
  * whatever xfer option has been taken.
- * GNOME_VFS_USE_UNIQUE_NAMES:  This is a check ot make sure that what you copy
+ * GNOME_VFS_USE_UNIQUE_NAMES:  This is a check to make sure that what you copy
  * onto the destination is not overwritten.  It will only copy the unique items
- * from the source to the destjnation.
+ * from the source to the destination.
  * GNOME_VFS_XFER_LINK_ITEMS: <TBA>
  * GNOME_VFS_XFER_TARGET_DEFAULT_PERMS: This means that the target file will
- * not have the same permissions of the source file, but will instead have the 
+ * not have the same permissions as of the source file, but will instead have the 
  * default permissions of the destination location.
  * @error_mode:  When this function returns you need to check the error code
  * for the results of the copy.  The results are generally:
  * GNOME_VFS_XFER_ERROR_MODE_ABORT: This means that the operation was aborted
  * by some sort of signal that interrupted the transfer.
  * GNOME_VFS_ERROR_MODE_QUERY: This means that no error has occured and that
- * you should query with the GNOME_VFS_XFER_PROGRESS_STATUS_VFSERROR.  See
+ * you should query with the GNOME_VFS_XFER_PROGRESS_STATUS_VFSERROR.
  * @overwrite_mode:  This sets the options to deal with data that are duplicate
  * between the source and the destination.  Your choices are:
  * GNOME_VFS_XFER_OVERWRITE_MODE_ABORT:  This means abort the transfer if you
@@ -2745,19 +2752,16 @@ gnome_vfs_xfer_uri_list (const GList *source_uri_list,
  * GNOME_VFS_XFER_OVERWRITE_MODE_REPLACE: Replace the files silently.  Don't
  * worry be happy.
  * GNOME_VFS_XFER_OVERWRITE_MODE_SKIP: Skip duplicate files silenty.
- * target.
- * @progress_callback:  This is an important call back because this is how you
- * communicate with your copy process.  
- * @data: Data to be want passed back in callbacks from the xfer engine
+ * @progress_callback:  This is an important callback because this is how you
+ * communicate with your transfer process.  
+ * @data: Data to be passed back to callbacks from the xfer engine.
  * 
- * This function will allow a person to copy data from one location to another.
- * The location is specified using a URIs as the means to describe the location
- * of the data.  Like any copy there are several options that can be set.
- * These can be set using the xfer_options.    In addition there are callback
- * mechanisms and error codes to provide feedback in the copy
- * process.
+ * This function will allow a person to transfer data from one location to another.
+ * The locations are specified using URIs. Several options can be set for the operation.
+ * These can be set using the @xfer_options. In addition, there are callback
+ * mechanisms and error codes to provide feedback in the transfer process.
  *
- * Return value: An integer representing the result of the operation.
+ * Return value: an integer representing the result of the operation.
  *
  **/
 GnomeVFSResult	
@@ -2791,15 +2795,15 @@ gnome_vfs_xfer_uri (const GnomeVFSURI *source_uri,
 
 /**
  * gnome_vfs_xfer_delete_list:
- * @source_uri_list:  This is a list containing uris
- * @error_mode:  Decide how you want to deal with interruptions
+ * @source_uri_list: This is a list containing uris.
+ * @error_mode: Decide how you want to deal with interruptions.
  * @xfer_options: Set whatever transfer options you need.
  * @progress_callback: Callback to check on progress of transfer.
- * @data: Data to be want passed back in callbacks from the xfer engine
+ * @data: Data to be passed back in callbacks from the xfer engine.
  *
- * Unlink items in the list @source_uri_list from their filesystems.
+ * Unlinks items in the @source_uri_list from their filesystems.
  *
- * Return value: %GNOME_VFS_OK if successful, or the appropriate error code otherwise
+ * Return value: %GNOME_VFS_OK if successful, or the appropriate error code otherwise.
  **/
 GnomeVFSResult 
 gnome_vfs_xfer_delete_list (const GList *source_uri_list, 
