@@ -60,6 +60,11 @@
 #include <fam.h>
 #endif
 
+#if defined(HAVE_LINUX_INOTIFY_H) || defined(HAVE_SYS_INOTIFY_H)
+#define USE_INOTIFY 1
+#include "inotify-helper.h"
+#endif
+
 #if HAVE_SYS_STATVFS_H
 #include <sys/statvfs.h>
 #endif
@@ -79,63 +84,6 @@
 #define DIR_SEPARATORS "/"
 #endif
 
-#if !GLIB_CHECK_VERSION (2,7,0)
-/* g_access lifted from GLib */
-#ifndef G_OS_WIN32
-#define g_access(filename, mode) access (filename, mode)
-#else
-static int
-g_access (const gchar *filename,
-	  int          mode)
-{
-#ifdef G_OS_WIN32
-  if (G_WIN32_HAVE_WIDECHAR_API ())
-    {
-      wchar_t *wfilename = g_utf8_to_utf16 (filename, -1, NULL, NULL, NULL);
-      int retval;
-      int save_errno;
-      
-      if (wfilename == NULL)
-	{
-	  errno = EINVAL;
-	  return -1;
-	}
-
-      retval = _waccess (wfilename, mode);
-      save_errno = errno;
-
-      g_free (wfilename);
-
-      errno = save_errno;
-      return retval;
-    }
-  else
-    {    
-      gchar *cp_filename = g_locale_from_utf8 (filename, -1, NULL, NULL, NULL);
-      int retval;
-      int save_errno;
-
-      if (cp_filename == NULL)
-	{
-	  errno = EINVAL;
-	  return -1;
-	}
-
-      retval = access (cp_filename, mode);
-      save_errno = errno;
-
-      g_free (cp_filename);
-
-      errno = save_errno;
-      return retval;
-    }
-#else
-  return access (filename, mode);
-#endif
-}
-#endif
-#endif
-
 #ifdef HAVE_FAM
 static FAMConnection *fam_connection = NULL;
 static gint fam_watch_id = 0;
@@ -146,7 +94,6 @@ typedef struct {
 	GnomeVFSURI *uri;
 	gboolean     cancelled;
 } FileMonitorHandle;
-
 #endif
 
 #ifdef PATH_MAX
@@ -1061,6 +1008,10 @@ do_read_directory (GnomeVFSMethod *method,
 		 */
 		return GNOME_VFS_OK;
 	}
+
+	if (handle->options & GNOME_VFS_FILE_INFO_GET_ACCESS_RIGHTS) {
+		get_access_info (file_info, full_name);
+	}
 	
 	if (handle->options & GNOME_VFS_FILE_INFO_GET_MIME_TYPE) {
 		get_mime_type (file_info, full_name, handle->options, &statbuf);
@@ -2003,8 +1954,14 @@ rename_helper (const gchar *old_full_name,
 	if (gnome_vfs_context_check_cancellation (context))
 		return GNOME_VFS_ERROR_CANCELLED;
 
+#ifdef G_OS_WIN32
+	if (force_replace && old_exists)
+		g_remove (new_full_name);
+#endif
+
 	retval = g_rename (old_full_name, new_full_name);
 
+#ifndef G_OS_WIN32
 	/* FIXME bugzilla.eazel.com 1186: The following assumes that,
 	 * if `new_uri' and `old_uri' are on different file systems,
 	 * `rename()' will always return `EXDEV' instead of `EISDIR',
@@ -2031,7 +1988,7 @@ rename_helper (const gchar *old_full_name,
 			retval = g_rename (old_full_name, new_full_name);
 		}
 	}
-
+#endif
 	if (retval != 0) {
 		return gnome_vfs_result_from_errno ();
 	}
@@ -2408,13 +2365,12 @@ monitor_setup (void)
 }
 #endif
 
-static GnomeVFSResult
-do_monitor_add (GnomeVFSMethod *method,
-		GnomeVFSMethodHandle **method_handle_return,
-		GnomeVFSURI *uri,
-		GnomeVFSMonitorType monitor_type)
-{
 #ifdef HAVE_FAM
+static GnomeVFSResult fam_monitor_add (GnomeVFSMethod *method,
+				       GnomeVFSMethodHandle **method_handle_return,
+				       GnomeVFSURI *uri,
+				       GnomeVFSMonitorType monitor_type)
+{
 	FileMonitorHandle *handle;
 	char *filename;
 
@@ -2457,16 +2413,56 @@ do_monitor_add (GnomeVFSMethod *method,
 	g_free (filename);
 
 	return GNOME_VFS_OK;
-#else
-	return GNOME_VFS_ERROR_NOT_SUPPORTED;
-#endif
+
 }
+#endif
+
+#ifdef USE_INOTIFY
+static GnomeVFSResult inotify_monitor_add (GnomeVFSMethod *method,
+					   GnomeVFSMethodHandle **method_handle_return,
+					   GnomeVFSURI *uri,
+					   GnomeVFSMonitorType monitor_type)
+{
+	ih_sub_t *sub;
+
+	sub = ih_sub_new (uri, monitor_type);
+	if (sub == NULL) {
+		return GNOME_VFS_ERROR_INVALID_URI;
+	}
+
+	if (ih_sub_add (sub) == FALSE) {
+		ih_sub_free (sub);
+		*method_handle_return = NULL;
+		return GNOME_VFS_ERROR_INVALID_URI;
+	}
+
+	*method_handle_return = (GnomeVFSMethodHandle *)sub;
+	return GNOME_VFS_OK;
+
+}
+#endif
 
 static GnomeVFSResult
-do_monitor_cancel (GnomeVFSMethod *method,
-		   GnomeVFSMethodHandle *method_handle)
+do_monitor_add (GnomeVFSMethod *method,
+		GnomeVFSMethodHandle **method_handle_return,
+		GnomeVFSURI *uri,
+		GnomeVFSMonitorType monitor_type)
 {
+#ifdef USE_INOTIFY
+	if (ih_startup ()) {
+		return inotify_monitor_add (method, method_handle_return, uri, monitor_type);
+	}
+#endif
 #ifdef HAVE_FAM
+	return fam_monitor_add (method, method_handle_return, uri, monitor_type);
+#endif
+	return GNOME_VFS_ERROR_NOT_SUPPORTED;
+}
+
+#ifdef HAVE_FAM
+static GnomeVFSResult fam_monitor_cancel (GnomeVFSMethod *method,
+					  GnomeVFSMethodHandle *method_handle)
+{
 	FileMonitorHandle *handle = (FileMonitorHandle *)method_handle;
 
 	if (!monitor_setup ()) {
@@ -2492,9 +2488,39 @@ do_monitor_cancel (GnomeVFSMethod *method,
 	G_UNLOCK (fam_connection);
 
 	return GNOME_VFS_OK;
-#else
-	return GNOME_VFS_ERROR_NOT_SUPPORTED;
+}
 #endif
+
+
+#ifdef USE_INOTIFY
+static GnomeVFSResult inotify_monitor_cancel (GnomeVFSMethod *method,
+					      GnomeVFSMethodHandle *method_handle)
+{
+	ih_sub_t *sub = (ih_sub_t *)method_handle;
+
+	if (sub->cancelled)
+		return GNOME_VFS_OK;
+
+	ih_sub_cancel (sub);
+	ih_sub_free (sub);
+	return GNOME_VFS_OK;
+
+}
+#endif
+
+static GnomeVFSResult
+do_monitor_cancel (GnomeVFSMethod *method,
+		   GnomeVFSMethodHandle *method_handle)
+{
+#ifdef USE_INOTIFY
+	if (ih_startup ()) {
+		return inotify_monitor_cancel (method, method_handle);
+	}
+#endif
+#ifdef HAVE_FAM
+	return fam_monitor_cancel (method, method_handle);
+#endif
+	return GNOME_VFS_ERROR_NOT_SUPPORTED;
 }
 
 static GnomeVFSResult
@@ -2591,7 +2617,7 @@ do_get_volume_free_space (GnomeVFSMethod *method,
 	*free_space = block_size * free_blocks;
 	
 	return GNOME_VFS_OK;
-#else // G_OS_WIN32
+#else /* G_OS_WIN32 */
 	g_warning ("Not implemented for WIN32: file-method.c:do_get_volume_free_space()");
 	return GNOME_VFS_ERROR_NOT_SUPPORTED;
 #endif
