@@ -33,6 +33,8 @@
 
 #include <sys/time.h>
 #include <arpa/inet.h>
+
+#include <gconf/gconf-client.h>
 #include <libgnomevfs/gnome-vfs-context.h>
 #include <libgnomevfs/gnome-vfs-inet-connection.h>
 #include <libgnomevfs/gnome-vfs-socket-buffer.h>
@@ -123,6 +125,8 @@ typedef struct {
 	gchar *server_type; /* the response from TYPE */
 	GnomeVFSResult fivefifty; /* the result to return for an FTP 550 */
 
+	const char *list_cmd; /* the command to be used for LIST */
+
 #ifdef HAVE_GSSAPI
 	gboolean use_gssapi;
 	gss_ctx_id_t gcontext;
@@ -140,9 +144,12 @@ typedef struct {
 	GnomeVFSFileInfoOptions file_info_options;
 } FtpDirHandle;
 
-
+/* Gconf key's "use_http_proxy" is for FTP as well, it should be renamed */
 static const char USE_PROXY_KEY[] = "/system/http_proxy/use_http_proxy";
 
+static const char PROXY_FTP_HOST_KEY[] = "/system/proxy/ftp_host";
+static const char PROXY_FTP_PORT_KEY[] = "/system/proxy/ftp_port";
+
 
 static GnomeVFSResult do_open	         (GnomeVFSMethod               *method,
 					  GnomeVFSMethodHandle         **method_handle,
@@ -172,8 +179,14 @@ static GnomeVFSResult ftp_connection_acquire   (GnomeVFSURI *uri,
 static void           ftp_connection_release   (FtpConnection *conn,
 						gboolean error_release);
 
+static GnomeVFSResult get_list_command (FtpConnection   *conn,
+					GnomeVFSContext *context);
+
 static const int   control_port = 21;
 
+/* FTP Proxy */
+static gchar *proxy_host = NULL;
+static int proxy_port = 0;
 
 /* A GHashTable of FtpConnectionPool */
 
@@ -245,6 +258,8 @@ ftp_response_to_vfs_result (FtpConnection *conn)
 	case 451:
 	case 551:
 	  return GNOME_VFS_ERROR_NOT_FOUND;
+	case 504:
+	  return GNOME_VFS_ERROR_BAD_PARAMETERS;
 	case 550:
 	  return conn->fivefifty;
 	case 452:
@@ -581,22 +596,42 @@ do_path_command (FtpConnection *conn,
 		 GnomeVFSURI *uri,
 		 GnomeVFSCancellation *cancellation) 
 {
-	char *path;
-	char *actual_command;
+	char *unescaped;
+	gchar *basename,
+	      *path,
+	      *actual_command,
+	      *cwd_command;
 	GnomeVFSResult result;
+	int end;
 
-	/* as some point we may need to make this execute a CD and then
-  	 * a command using the basename rather than the full path. I am yet
-	 * to come across such a system.
-	 */
-	path = gnome_vfs_unescape_string (uri->text, G_DIR_SEPARATOR_S);
-
-	if (path == NULL || path[0] == '\0') {
-		actual_command = g_strconcat (command, " /", NULL);
-	} else {
-		actual_command = g_strconcat (command, " ", path, NULL);
+	/* Execute a CD and then a command using the basename rather
+	 * than the full path. Useful for some (buggy?) systems. */
+	unescaped = gnome_vfs_unescape_string (uri->text, G_DIR_SEPARATOR_S);
+	if (unescaped == NULL || unescaped[0] == '\0') {
+		unescaped = g_strdup ("/");
 	}
+
+	/* Remove trailing slashes */
+	end = strlen(unescaped) - 1;
+	if (end > 0 && unescaped[end] == '/') {
+		unescaped[end] = 0;
+	}
+	
+	basename = g_path_get_basename (unescaped);
+	path = g_path_get_dirname (unescaped);
+	g_free (unescaped);
+
+	cwd_command = g_strconcat ("CWD ", path, NULL);
 	g_free (path);
+	result = do_basic_command (conn, cwd_command, cancellation);
+	g_free (cwd_command);
+	if (result != GNOME_VFS_OK) {
+		g_free (basename);
+		return result;
+	}
+
+	actual_command = g_strconcat (command, " ", basename, NULL);
+	g_free (basename);
 
 	result = do_basic_command (conn, actual_command, cancellation);
 	g_free (actual_command);
@@ -729,22 +764,41 @@ do_transfer_command (FtpConnection *conn, gchar *command, GnomeVFSContext *conte
 static GnomeVFSResult
 do_path_transfer_command (FtpConnection *conn, gchar *command, GnomeVFSURI *uri, GnomeVFSContext *context) 
 {
-	char *path;
-	char *actual_command;
+	char *unescaped;
+	gchar *basename,
+	      *path,
+	      *cwd_command,
+	      *actual_command;
 	GnomeVFSResult result;
+	int end;
 
-	/* as some point we may need to make this execute a CD and then
-  	 * a command using the basename rather than the full path. I am yet
-	 * to come across such a system.
-	 */
-	path = gnome_vfs_unescape_string (uri->text, G_DIR_SEPARATOR_S);
-
-	if (path == NULL || path[0] == '\0') {
-		actual_command = g_strconcat (command, " /", NULL);
-	} else {
-		actual_command = g_strconcat (command, " ", path, NULL);
+	unescaped = gnome_vfs_unescape_string (uri->text, G_DIR_SEPARATOR_S);
+	if (unescaped == NULL || unescaped[0] == '\0') {
+		unescaped = g_strdup ("/");
 	}
+
+	/* Remove trailing slashes */
+	end = strlen(unescaped) - 1;
+	if (end > 0 && unescaped[end] == '/') {
+		unescaped[end] = 0;
+	}
+
+	basename = g_path_get_basename (unescaped);
+	path = g_path_get_dirname (unescaped);
+	g_free (unescaped);
+
+	cwd_command = g_strconcat ("CWD ", path, NULL);
 	g_free (path);
+	result = do_basic_command (conn, cwd_command,
+				   get_cancellation (context));
+	g_free (cwd_command);
+	if (result != GNOME_VFS_OK) {
+		g_free (basename);
+		return result;
+	}
+
+	actual_command = g_strconcat (command, " ", basename, NULL);
+	g_free (basename);
 
 	result = do_transfer_command (conn, actual_command, context);
 	g_free (actual_command);
@@ -802,7 +856,8 @@ save_authn_info (GnomeVFSURI *uri,
 
 static gboolean
 query_user_for_authn_info (GnomeVFSURI *uri, 
-			   char **user, char **pass, char **keyring, gboolean *save,
+			   char **user, char **pass, char **keyring,
+			   gboolean *save, gboolean *aborted,
 			   gboolean no_username)
 {
 	GnomeVFSModuleCallbackFullAuthenticationIn in_args;
@@ -843,10 +898,10 @@ query_user_for_authn_info (GnomeVFSURI *uri,
 		*pass = g_strdup ("nobody@gnome.org");
                 goto error;
         }
-	
-        ret = !out_args.abort_auth;
-	
-        if (!ret) {
+
+	*aborted = out_args.abort_auth;
+
+	if (out_args.abort_auth) {
                 goto error;
         }
 	
@@ -875,8 +930,8 @@ query_user_for_authn_info (GnomeVFSURI *uri,
        	g_free (out_args.domain);
        	g_free (out_args.password);
        	g_free (out_args.keyring);
-	
-	return ret;
+
+	return ret && !out_args.abort_auth;
 } 
 
 static gboolean
@@ -1063,7 +1118,15 @@ ftp_login (FtpConnection *conn,
 	gchar *tmpstring;
 	GnomeVFSResult result;
 	
-	tmpstring = g_strdup_printf ("USER %s", user);
+	if (!proxy_host) {
+		tmpstring = g_strdup_printf ("USER %s", user);
+	} else {
+		/* Using FTP proxy */
+		tmpstring = g_strdup_printf ("USER %s@%s",
+					     user,
+					     gnome_vfs_uri_get_host_name (conn->uri));
+	}
+
 	result = do_basic_command (conn, tmpstring, cancellation);
 	g_free (tmpstring);
 
@@ -1087,7 +1150,9 @@ try_connection (GnomeVFSURI *uri,
 	gint port = control_port;
 	const char *host;
 	
-	if (gnome_vfs_uri_get_host_port (uri)) {
+	if (proxy_host) {
+		port = proxy_port;
+	} else if (gnome_vfs_uri_get_host_port (uri)) {
                 port = gnome_vfs_uri_get_host_port (uri);
         }
 
@@ -1095,7 +1160,12 @@ try_connection (GnomeVFSURI *uri,
 	if (*saved_ip != NULL) {
 		host = *saved_ip;
 	} else {
-		host = gnome_vfs_uri_get_host_name (uri);
+		if (!proxy_host) {
+			host = gnome_vfs_uri_get_host_name (uri);
+		} else  {
+			/* Use FTP proxy */
+			host = proxy_host;
+		}
 	}
 
 	if (host == NULL) {
@@ -1242,6 +1312,7 @@ ftp_connection_create (FtpConnectionPool *pool,
 	gboolean got_connection;
 	gboolean ret;
 	gboolean connection_failed;
+	gboolean aborted = FALSE;
 	
 	cancellation = get_cancellation (context);
 	
@@ -1313,7 +1384,7 @@ ftp_connection_create (FtpConnectionPool *pool,
 				pool->num_connections++;
 				G_UNLOCK (connection_pools);
 				ret = query_user_for_authn_info (uri, &user, &pass, &keyring, &save_authn, 
-								 !uri_has_username); 
+								 &aborted, !uri_has_username); 
 				G_LOCK (connection_pools);
 				pool->num_connections--;
 				if (!ret) {
@@ -1323,7 +1394,12 @@ ftp_connection_create (FtpConnectionPool *pool,
 					g_free (user);
 					g_free (pass);
 					g_free (keyring);
-                                	return GNOME_VFS_ERROR_LOGIN_FAILED;
+
+					if (aborted) {
+						return GNOME_VFS_ERROR_CANCELLED;
+					} else {
+						return GNOME_VFS_ERROR_LOGIN_FAILED;
+					}
 				}
 				g_string_free (conn->response_buffer, TRUE);
                         	conn->response_buffer = g_string_new ("");
@@ -2283,6 +2359,50 @@ do_get_file_info_from_handle (GnomeVFSMethod *method,
 				 file_info, options, context);
 }
 
+/* Tries out all suitable LIST parameter combinations until one succeeds.
+ * If there is an unexpected error, it is returned and has to be handled
+ * by the caller. Note that a successful LIST command also means that the
+ * list is actually requested, which also has to be handled by the caller.
+ * */
+static GnomeVFSResult
+get_list_command (FtpConnection   *conn,
+		  GnomeVFSContext *context)
+{
+	GnomeVFSResult result;
+
+	static const char * const osx_candidates[] = { "LIST -a", NULL };
+	static const char * const non_osx_candidates[] = { "LIST -aL", "LIST -a", "LIST -L", NULL };
+
+	const char * const *candidates;
+
+	if (strstr(conn->server_type, "MACOS")) {
+		candidates = osx_candidates;
+	} else {
+		candidates = non_osx_candidates;
+	}
+
+	do {
+		result = do_transfer_command (conn, (char *) *candidates, context);
+	}
+	while (*++candidates != NULL &&
+	       result == GNOME_VFS_ERROR_BAD_PARAMETERS);
+
+	if (result == GNOME_VFS_OK) {
+		/* we found the best suitable candidate for this server. */
+		g_assert (candidates != NULL);
+
+		conn->list_cmd = *candidates;
+	} else {
+		/* all commands failed, or an unexpected error occured,
+		 * just use "LIST". */
+
+		result = do_transfer_command (conn, "LIST", context);
+		conn->list_cmd = "LIST";
+	}
+
+	return result;
+}
+
 static GnomeVFSResult
 do_open_directory (GnomeVFSMethod *method,
 		   GnomeVFSMethodHandle **method_handle,
@@ -2343,14 +2463,14 @@ do_open_directory (GnomeVFSMethod *method,
 	result = do_path_command (conn, "CWD", uri, cancellation);
 	if (result != GNOME_VFS_OK) {
 		ftp_connection_release (conn, TRUE);
+		g_string_free (dirlist, TRUE);
 		return result;
 	}
 
-	if(strstr(conn->server_type,"MACOS")) {
-		/* don't ask for symlinks from MacOS servers */
-		result = do_transfer_command (conn, "LIST", context);
+	if (conn->list_cmd != NULL) {
+		result = do_transfer_command (conn, (char *) conn->list_cmd, context);
 	} else {
-		result = do_transfer_command (conn, "LIST -L", context);
+		result = get_list_command (conn, context);
 	}
 
 	if (result != GNOME_VFS_OK) {
@@ -2762,12 +2882,37 @@ GnomeVFSMethod *
 vfs_module_init (const char *method_name, 
 		 const char *args)
 {
+	GConfClient *gclient;
+
 	connection_pools = g_hash_table_new (ftp_connection_uri_hash, 
 					     ftp_connection_uri_equal);
+
+	gclient = gconf_client_get_default ();
+	if (gclient) {
+		if (gconf_client_get_bool (gclient, USE_PROXY_KEY, NULL)) {
+			/* Using FTP proxy */
+			proxy_host = gconf_client_get_string (gclient,
+							      PROXY_FTP_HOST_KEY,
+							      NULL);
+			/* Don't use blank hostname */
+			if (proxy_host &&
+			    *proxy_host == 0) {
+				g_free (proxy_host);
+				proxy_host = NULL;
+			}
+			proxy_port = gconf_client_get_int (gclient,
+							   PROXY_FTP_PORT_KEY,
+							   NULL);
+		} else proxy_host = NULL;
+	}
+
 	return &method;
 }
 
 void
 vfs_module_shutdown (GnomeVFSMethod *method)
 {
+	if (proxy_host) {
+		g_free (proxy_host);
+	}
 }
