@@ -69,7 +69,7 @@
 #include <glib/gi18n-lib.h>
 
 #include "sftp.h"
-#include "pty-open.h"
+#include <libgnomevfs/gnome-vfs-pty.h>
 
 size_t default_req_len = 32768;
 guint max_req = 16;
@@ -485,10 +485,13 @@ buffer_read_file_info (Buffer *buf, GnomeVFSFileInfo *info)
 		info->valid_fields |= GNOME_VFS_FILE_INFO_FIELDS_SIZE;
 		info->size = buffer_read_gint64 (buf);
 	}
+	
 	if (flags & SSH2_FILEXFER_ATTR_UIDGID) {
 		info->uid = buffer_read_gint32 (buf);
 		info->gid = buffer_read_gint32 (buf);
+		info->valid_fields |= GNOME_VFS_FILE_INFO_FIELDS_IDS;	
 	}
+	
 	if (flags & SSH2_FILEXFER_ATTR_PERMISSIONS) {
 		info->valid_fields |= GNOME_VFS_FILE_INFO_FIELDS_PERMISSIONS;
 		info->permissions = buffer_read_gint32 (buf);
@@ -521,10 +524,6 @@ buffer_read_file_info (Buffer *buf, GnomeVFSFileInfo *info)
 	info->valid_fields |= GNOME_VFS_FILE_INFO_FIELDS_IO_BLOCK_SIZE;
 	info->io_block_size = default_req_len;
 
-	/* Set the uid and gid fields naively to the current user to trick Nautilus */
-	info->uid = getuid ();
-	info->gid = getgid ();
-
 	DEBUG4 (g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "Read file info from %p", buf));
 }
 
@@ -927,7 +926,8 @@ invoke_full_auth (const GnomeVFSURI *uri,
 		  char **user_out,
 		  char **object_out,
 		  char **authtype_out,
-		  gboolean *save_password_out)
+		  gboolean *save_password_out,
+		  gboolean *abort_auth_out)
 {
 	GnomeVFSModuleCallbackFullAuthenticationIn in_args;
 	GnomeVFSModuleCallbackFullAuthenticationOut out_args;
@@ -970,6 +970,8 @@ invoke_full_auth (const GnomeVFSURI *uri,
 		*password_out = NULL;
 	}
 
+	*abort_auth_out = out_args.abort_auth;
+
 	g_free (in_args.uri);
 	g_free (in_args.username);
 	g_free (in_args.object);
@@ -1146,7 +1148,7 @@ sftp_connect (SftpConnection **connection, const GnomeVFSURI *uri)
 
 	tty_fd = -1;
 #ifdef USE_PTY
-	tty_fd =_gnome_vfs_pty_open(&ssh_pid, NULL,
+	tty_fd = gnome_vfs_pty_open(&ssh_pid, GNOME_VFS_PTY_REAP_CHILD, NULL,
 				    args[0], args+1, NULL,
 				    300, 300, 
 				    &out_fd, &in_fd, &err_fd);
@@ -1208,6 +1210,7 @@ sftp_connect (SftpConnection **connection, const GnomeVFSURI *uri)
 		char *endpos;
 		char *hostname = NULL;
 		char *fingerprint = NULL;
+		gboolean aborted;
 
 		if (client_vendor == SFTP_VENDOR_SSH) {
 			prompt_fd = err_fd;
@@ -1255,11 +1258,14 @@ sftp_connect (SftpConnection **connection, const GnomeVFSURI *uri)
 					g_io_channel_write_chars (tty_channel, "\n", 1, &len, NULL);
 					g_io_channel_flush (tty_channel, NULL);
 				} else if (invoke_full_auth (uri, done_auth, buffer, &password, &keyring, 
-							     &user, &object, &authtype, &save_password) && password != NULL) {
+							     &user, &object, &authtype, &save_password, &aborted) && password != NULL) {
 					full_auth = TRUE;
 					g_io_channel_write_chars (tty_channel, password, -1, &len, NULL);
 					g_io_channel_write_chars (tty_channel, "\n", 1, &len, NULL);
 					g_io_channel_flush (tty_channel, NULL);
+				} else if (aborted) {
+					res = GNOME_VFS_ERROR_CANCELLED;
+					goto bail;
 				} else {
 					res = GNOME_VFS_ERROR_ACCESS_DENIED;
 					goto bail;
@@ -1752,7 +1758,7 @@ do_open (GnomeVFSMethod        *method,
 	guint id;
 	gint32 sftp_mode;
 	gchar *sftp_handle;
-	gint sftp_handle_len, sftp_res;
+	gint sftp_handle_len;
 	gchar *path;
 
 	DEBUG (g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "%s: Enter", __FUNCTION__));
@@ -1788,9 +1794,9 @@ do_open (GnomeVFSMethod        *method,
 	buffer_send (&msg, conn->out_fd);
 	buffer_free (&msg);
 	
-	sftp_res = iobuf_read_handle (conn->in_fd, &sftp_handle, id, (guint32 *)&sftp_handle_len);
+	res = iobuf_read_handle (conn->in_fd, &sftp_handle, id, (guint32 *)&sftp_handle_len);
 
-	if (sftp_res == SSH2_FX_OK) {
+	if (res == GNOME_VFS_OK) {
 		handle = g_new0 (SftpOpenHandle, 1);
 		handle->sftp_handle = sftp_handle;
 		handle->sftp_handle_len = sftp_handle_len;
@@ -1808,7 +1814,7 @@ do_open (GnomeVFSMethod        *method,
 		sftp_connection_unlock (conn);
 
 		DEBUG (g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "%s: Exit", __FUNCTION__));
-		return sftp_status_to_vfs_result (sftp_res);
+		return res;
 	}
 }
 
@@ -1829,7 +1835,6 @@ do_create (GnomeVFSMethod        *method,
 	Buffer msg;
 	int id;
 	int ssh_mode;
-	guint sftp_res;
 
 	gchar *sftp_handle;
 	guint sftp_handle_len;
@@ -1875,10 +1880,10 @@ do_create (GnomeVFSMethod        *method,
 	buffer_send (&msg, conn->out_fd);
 	buffer_free (&msg);
 
-	sftp_res = iobuf_read_handle (conn->in_fd, &sftp_handle, id, &sftp_handle_len);
+	res = iobuf_read_handle (conn->in_fd, &sftp_handle, id, &sftp_handle_len);
 
 
-	if (sftp_res == SSH2_FX_OK) {
+	if (res == GNOME_VFS_OK) {
 		handle = g_new0 (SftpOpenHandle, 1);
 		handle->sftp_handle = sftp_handle;
 		handle->sftp_handle_len = sftp_handle_len;
@@ -1896,7 +1901,7 @@ do_create (GnomeVFSMethod        *method,
 		sftp_connection_unlock (conn);
 		
 		DEBUG (g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "%s: Exit", __FUNCTION__));
-		return sftp_status_to_vfs_result (sftp_res);
+		return res;
 	}
 }
 
@@ -2414,7 +2419,6 @@ do_open_directory (GnomeVFSMethod          *method,
 	gchar *sftp_handle;
 	Buffer msg;
 	guint id, sftp_handle_len;
-	guint sftp_res;
 	gchar *path;
 
 	DEBUG (g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "%s: Enter", __FUNCTION__));
@@ -2446,11 +2450,11 @@ do_open_directory (GnomeVFSMethod          *method,
 
 	buffer_free (&msg);
 
-	sftp_res = iobuf_read_handle (conn->in_fd, &sftp_handle, id, &sftp_handle_len);
+	res = iobuf_read_handle (conn->in_fd, &sftp_handle, id, &sftp_handle_len);
 
-	DEBUG (g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "%s: Result is %d", __FUNCTION__, sftp_res));
+	DEBUG (g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "%s: Result is %d", __FUNCTION__, res));
 
-	if (sftp_res == SSH2_FX_OK) {
+	if (res == GNOME_VFS_OK) {
 		handle = g_new0 (SftpOpenHandle, 1);
 		handle->sftp_handle = sftp_handle;
 		handle->sftp_handle_len = sftp_handle_len;
@@ -2469,15 +2473,15 @@ do_open_directory (GnomeVFSMethod          *method,
 		return GNOME_VFS_OK;
 	} else {
 		/* For some reason, some servers report EOF when the directory doesn't exist. *shrug* */
-		if (sftp_res == SSH2_FX_EOF)
-			sftp_res = SSH2_FX_NO_SUCH_FILE;
+		if (res == GNOME_VFS_ERROR_EOF)
+			res = GNOME_VFS_ERROR_NOT_FOUND;
 
 		sftp_connection_unref (conn);
 		sftp_connection_unlock (conn);
 
 		*method_handle = NULL;
 		DEBUG (g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "%s: Exit", __FUNCTION__));
-		return sftp_status_to_vfs_result (sftp_res);
+		return res;
 	}
 }
 
